local SM = LibStub:GetLibrary("LibSharedMedia-2.0")
local Graph = LibStub:GetLibrary("LibGraph-2.0")
-- local Graph = AceLibrary("Graph-1.0")

local L = AceLibrary("AceLocale-2.2"):new("Recount")
local BC = AceLibrary("Babble-Class-2.2")

-- Elsia: Note, most strings here haven't been localized. Need to grab all button and text labels here and put into localization registration.
-- Just started with the color selection ones to give an example. See Recount.lua.

local me={}

local SavedCheckVars={}

local EditableColors={
	["Window"]={
		"Title",
		"Background",
		"Title Text",
	},
	["Other Windows"]={
		"Title",
		"Background",
		"Title Text",
	},
	["Bar"]={
		"Bar Text",
	},
	["Class"]={
		"Druid",
		"Hunter",
		"Mage",
		"Paladin",
		"Priest",
		"Rogue",
		"Shaman",
		"Warlock",
		"Warrior",
		"Pet",
		"Mob",
	}
}

local ClassStrings={
	["DRUID"]=true,
	["HUNTER"]=true,
	["MAGE"]=true,
	["PALADIN"]=true,
	["PRIEST"]=true,
	["ROGUE"]=true,
	["SHAMAN"]=true,
	["WARLOCK"]=true,
	["WARRIOR"]=true,
	["PET"]=false, -- Elsia: These two are not supported by RAID_CLASS_COLORS or Babble-Class
	["MOB"]=false,
}

function me:LBC(Name) -- Allow localization of unit strings via Babble-Class
	local CName = string.upper(Name)
	if ClassStrings[CName] then -- Elsia: Only Babble what babble knows
		return BC[Name]
	else
		return L[Name]
	end
end

function Recount:FixUnitString(Name) -- This is to handle caps of default unit strings
	local CName = string.upper(Name)
	if ClassStrings[CName]~=nil then -- Elsia: Caps all unit strings
		return CName
	else
		return Name
	end
end

function Recount:ResetDefaultWindowColors()
	Recount.Colors:SetColor("Window", "Title", { r = 1, g = 0, b = 0, a = 1})
	Recount.Colors:SetColor("Window", "Background", { r = 24/255, g = 24/255, b = 24/255, a = 1})
	Recount.Colors:SetColor("Window", "Title Text", { r = 1, g = 1, b = 1, a = 1})
	Recount.Colors:SetColor("Other Windows", "Title", { r = 1, g = 0, b = 0, a = 1})
	Recount.Colors:SetColor("Other Windows", "Background", { r = 24/255, g = 24/255, b = 24/255, a = 1})
	Recount.Colors:SetColor("Other Windows", "Title Text", { r = 1, g = 1, b = 1, a = 1})
end

function Recount:ResetDefaultClassColors()
	for k,v in pairs(EditableColors.Class) do
		v = Recount:FixUnitString(v)
		if v=="PET" then
			Recount.Colors:SetColor("Class", "PET", { r = 0.09, g = 0.61, b = 0.55 })
		elseif v=="MOB" then
			Recount.Colors:SetColor("Class", "MOB", { r = 0.58, g = 0.24, b = 0.63 })
		else
			Recount.db.profile.Colors["Class"][v]=nil -- Elsia: For the rest deleting and relying on GetColor to do the job works fine. Odd.
			Recount.Colors:UpdateColor("Class",v)
		end
	end
	Recount.Colors:SetColor("Bar", "Bar Text", { r = 1, g = 1, b = 1})
end


function me:SetColorRow(Branch,Name)
	self.Branch=Branch
	self.Text:SetText(me:LBC(Name))
	Name = Recount:FixUnitString(Name)
	self.Name=Name
	Recount.Colors:UnregisterItem(self.Background)
	Recount.Colors:UnregisterItem(self.Key)
	Recount.Colors:RegisterTexture(Branch,Name,self.Background)
	Recount.Colors:RegisterTexture(Branch,Name,self.Key)
end

function me:CreateColorRow(parent, frame)
	local theFrame=CreateFrame("Frame",nil,parent)

	theFrame:SetWidth(190)
	theFrame:SetHeight(13)

	theFrame.Background=theFrame:CreateTexture(nil,"BACKGROUND")
	theFrame.Background:SetAllPoints(theFrame)
	theFrame.Background:SetTexture(1,1,1,0.3)
	theFrame.Background:Hide()

	theFrame.Key=theFrame:CreateTexture(nil,"OVERLAY")
	theFrame.Key:SetHeight(13)
	theFrame.Key:SetWidth(13)
	theFrame.Key:SetPoint("LEFT",theFrame,"LEFT",0,0)
	theFrame.Key:SetTexture(1,1,1)

	theFrame.Text=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Text:SetPoint("LEFT",theFrame,"LEFT",16,0)

	theFrame:EnableMouse(true)
	theFrame:SetScript("OnEnter",function() theFrame.Background:Show() end)
	theFrame:SetScript("OnLeave",function() theFrame.Background:Hide() end)
	theFrame:SetScript("OnMouseDown",function() Recount.Colors:EditColor(this.Branch,this.Name,me.ConfigWindow) end)
	theFrame.SetRow=me.SetColorRow

	return theFrame
end

function me:CreateWindowColorSelection(parent)
	me.WindowColorOptions=CreateFrame("Frame",nil,parent)

	local theFrame=me.WindowColorOptions

	theFrame:SetHeight(200)
	theFrame:SetWidth(200)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",0,-34)

	theFrame.Rows={}

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)
	theFrame.Title:SetText("Window Color Selection")

	local i=1
	theFrame.MainWindowTitle=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.MainWindowTitle:SetPoint("TOP",theFrame,"TOP",0,-4-i*14)
	theFrame.MainWindowTitle:SetText("Main Window")
	i=i+1
	for k,v in pairs(EditableColors.Window) do
		theFrame.Rows[i]=me:CreateColorRow(theFrame)
		theFrame.Rows[i]:SetRow("Window",v)
		theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",4,-2-i*14)
		i=i+1
		if i>16 then
			return
		end
	end
	theFrame.DetailWindowTitle=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.DetailWindowTitle:SetPoint("TOP",theFrame,"TOP",0,-4-i*14)
	theFrame.DetailWindowTitle:SetText("Other Windows")
	i=i+1
	for k,v in pairs(EditableColors.Window) do
		theFrame.Rows[i]=me:CreateColorRow(theFrame)
		theFrame.Rows[i]:SetRow("Other Windows",v)
		theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",4,-4-i*14)
		i=i+1
		if i>16 then
			return
		end
	end

	theFrame.ResetColButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.ResetColButton:SetWidth(120)
	theFrame.ResetColButton:SetHeight(18)
	theFrame.ResetColButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",40,-210)
	theFrame.ResetColButton:SetScript("OnClick",function() Recount:ResetDefaultWindowColors() end)
	theFrame.ResetColButton:SetText("Reset Colors")
end

function me:CreateClassColorSelection(parent)
	me.ClassColorOptions=CreateFrame("Frame",nil,parent)

	local theFrame=me.ClassColorOptions

	theFrame:SetHeight(200)
	theFrame:SetWidth(200)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",200,-34)

	theFrame.Rows={}

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)
	theFrame.Title:SetText("Bar Color Selection")
	
	local i=1
	--theFrame.ClassTitle=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	--theFrame.ClassTitle:SetPoint("TOP",theFrame,"TOP",0,-2-i*14)
	--theFrame.ClassTitle:SetText("Bar Font Color")
	--i=i+1
	for k,v in pairs(EditableColors.Bar) do
		theFrame.Rows[i]=me:CreateColorRow(theFrame)
		theFrame.Rows[i]:SetRow("Bar",v)
		theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",4,-4-i*14)
		i=i+1
		if i>16 then
			return
		end
	end
	
	theFrame.ClassTitle=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.ClassTitle:SetPoint("TOP",theFrame,"TOP",0,-6-i*14)
	theFrame.ClassTitle:SetText("Class Colors")
	i=i+1
	for k,v in pairs(EditableColors.Class) do
		theFrame.Rows[i]=me:CreateColorRow(theFrame)
		theFrame.Rows[i]:SetRow("Class",v)
		theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",4,-6-i*14)
		i=i+1
		if i>16 then
			return
		end
	end

	i=i+1
	theFrame.ResetColButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.ResetColButton:SetWidth(120)
	theFrame.ResetColButton:SetHeight(18)
	theFrame.ResetColButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",40,-210)
	theFrame.ResetColButton:SetScript("OnClick",function() Recount:ResetDefaultClassColors() end)
	theFrame.ResetColButton:SetText("Reset Colors")
end

function me:CreateIconFrame(parent,texture,title,text)
	local theFrame=CreateFrame("Frame",nil,parent)
	theFrame:SetWidth(18)
	theFrame:SetHeight(18)

	theFrame.texture=theFrame:CreateTexture(nil,"OVERLAY")
	theFrame.texture:SetAllPoints(theFrame)
	theFrame.texture:SetTexture(texture)
	theFrame.title=title
	theFrame.text=text

	theFrame:SetScript("OnEnter",function()
					GameTooltip:SetOwner(theFrame, "ANCHOR_TOPRIGHT")
					GameTooltip:ClearLines()
					GameTooltip:AddLine(theFrame.title)
					GameTooltip:AddLine(theFrame.text,1,1,1,true)
					GameTooltip:Show()
				     end)
	theFrame:SetScript("OnLeave",function() GameTooltip:Hide() end)

	theFrame:EnableMouse()
	theFrame:Show()

	return theFrame
end

function me:ConfigureCheckbox(check)
	check:SetWidth(20)
	check:SetHeight(20)
	check:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true) else this:SetChecked(false) end end)
	check:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Up")
	check:SetPushedTexture("Interface\\Buttons\\UI-CheckBox-Down")
	check:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled")
	check:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check")
	check:SetHighlightTexture("Interface\\Buttons\\UI-CheckBox-Highlight")
end

function me:CreateSavedCheckbox(Text, parent,VarTop,VarName)
	local Checkbox=CreateFrame("CheckButton",nil,parent)
	me:ConfigureCheckbox(Checkbox)

	Checkbox.Text=Checkbox:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Checkbox.Text:SetText(Text)
	Checkbox.Text:SetPoint("LEFT",Checkbox,"RIGHT",8,0)

	SavedCheckVars[#SavedCheckVars+1]={Checkbox,VarTop,VarName}

	return Checkbox
end

function me:CreateFilterRow(parent,label,header)
	local theFrame=CreateFrame("Frame",nil,parent)

	theFrame:SetWidth(196)
	theFrame:SetHeight(16)
	
	theFrame.Label=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	if not header then
		theFrame.Label:SetTextColor(1,1,1,1)
	end
	theFrame.Label:SetText(" "..label)
	theFrame.Label:SetPoint("LEFT",theFrame,"LEFT",0,0)

	theFrame.ShowData=CreateFrame("CheckButton",nil,theFrame)
	me:ConfigureCheckbox(theFrame.ShowData)
	theFrame.ShowData:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true);  else this:SetChecked(false); end  me:SaveFilterConfig()  end)

	theFrame.RecordData=CreateFrame("CheckButton",nil,theFrame)
	me:ConfigureCheckbox(theFrame.RecordData)
	theFrame.RecordData:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true);theFrame.RecordTime:Enable();theFrame.TrackDeaths:Enable();theFrame.TrackBuffs:Enable() else this:SetChecked(false);theFrame.RecordTime:Disable();theFrame.TrackDeaths:Disable();theFrame.TrackBuffs:Disable() end me:SaveFilterConfig() end)


	theFrame.RecordTime=CreateFrame("CheckButton",nil,theFrame)	
	me:ConfigureCheckbox(theFrame.RecordTime)
	theFrame.RecordTime:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true);  else this:SetChecked(false); end  me:SaveFilterConfig()  end)

	theFrame.TrackDeaths=CreateFrame("CheckButton",nil,theFrame)
	me:ConfigureCheckbox(theFrame.TrackDeaths)
	theFrame.TrackDeaths:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true);  else this:SetChecked(false); end  me:SaveFilterConfig()  end)

	theFrame.TrackBuffs=CreateFrame("CheckButton",nil,theFrame)
	me:ConfigureCheckbox(theFrame.TrackBuffs)
	theFrame.TrackBuffs:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true);  else this:SetChecked(false); end  me:SaveFilterConfig()  end)


	theFrame.ShowData:SetPoint("RIGHT",theFrame.RecordData,"LEFT",0,0)
	theFrame.RecordData:SetPoint("RIGHT",theFrame.RecordTime,"LEFT",0,0)
	theFrame.RecordTime:SetPoint("RIGHT",theFrame.TrackDeaths,"LEFT",0,0)
	theFrame.TrackDeaths:SetPoint("RIGHT",theFrame.TrackBuffs,"LEFT",0,0)
	theFrame.TrackBuffs:SetPoint("RIGHT",theFrame,"RIGHT",-1,0)

	theFrame.ShowData:Show()
	theFrame.RecordData:Show()
	theFrame.RecordTime:Show()
	theFrame.TrackDeaths:Show()
	theFrame.TrackBuffs:Show()

	return theFrame
end

function me:SetupFilterOptions(parent)
	me.FilterOptions=CreateFrame("Frame",nil,parent)
	local theFrame=me.FilterOptions

	theFrame:SetHeight(196)
	theFrame:SetWidth(196)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",2,-34)

	theFrame.Title_Show=me:CreateIconFrame(theFrame,"Interface/Icons/INV_Misc_Eye_01","Show","Is this shown in the main window?")
	theFrame.Title_Data=me:CreateIconFrame(theFrame,"Interface/Icons/INV_Misc_Note_02","Record Data","Whether data is recorded for this type")
	theFrame.Title_Time=me:CreateIconFrame(theFrame,"Interface/Icons/INV_Misc_PocketWatch_02","Record Time Data","Whether time data is recorded for this type (used for graphs can be a |cffff2020memory hog|r if you are concerned about memory)")
	theFrame.Title_Deaths=me:CreateIconFrame(theFrame,"Interface/Icons/Ability_Creature_Cursed_02","Record Deaths","Records when deaths occur and the past few actions involving this type")
	theFrame.Title_Buffs=me:CreateIconFrame(theFrame,"Interface/Icons/Ability_Warrior_SavageBlow","Record Buffs/Debuffs","Records the times and applications of buff/debuffs on this type")
	
	theFrame.Title_Show:SetPoint("RIGHT",theFrame.Title_Data,"LEFT",-2,0)
	theFrame.Title_Data:SetPoint("RIGHT",theFrame.Title_Time,"LEFT",-2,0)
	theFrame.Title_Time:SetPoint("RIGHT",theFrame.Title_Deaths,"LEFT",-2,0)
	theFrame.Title_Deaths:SetPoint("RIGHT",theFrame.Title_Buffs,"LEFT",-2,0)
	theFrame.Title_Buffs:SetPoint("TOPRIGHT",theFrame,"TOPRIGHT",-2,-2)

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetText("Filters")
	theFrame.Title:SetPoint("TOPLEFT",theFrame,"TOPLEFT",2,-4)



	theFrame.Filters={}
	local Filters=theFrame.Filters

	theFrame.TitlePlayers=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.TitlePlayers:SetText(" Players")
	theFrame.TitlePlayers:SetPoint("TOPLEFT",theFrame,"TOPLEFT",0,-26)

	Filters.Self=me:CreateFilterRow(theFrame,"  Self")
	Filters.Self:SetPoint("TOPLEFT",theFrame.TitlePlayers,"BOTTOMLEFT",0,-1)

	Filters.Grouped=me:CreateFilterRow(theFrame,"  Grouped")
	Filters.Grouped:SetPoint("TOPLEFT",Filters.Self,"BOTTOMLEFT",0,-1)

	Filters.Ungrouped=me:CreateFilterRow(theFrame,"  Ungrouped")
	Filters.Ungrouped:SetPoint("TOP",Filters.Grouped,"BOTTOM",0,-1)


	Filters.Pet=me:CreateFilterRow(theFrame,"Pets",true)
	Filters.Pet:SetPoint("TOP",Filters.Ungrouped,"BOTTOM",0,-1)

	theFrame.TitleMobs=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.TitleMobs:SetText(" Mobs")
	theFrame.TitleMobs:SetPoint("TOPLEFT",Filters.Pet,"BOTTOMLEFT",0,-1)

	Filters.Trivial=me:CreateFilterRow(theFrame,"  Trivial")
	Filters.Trivial:SetPoint("TOPLEFT",theFrame.TitleMobs,"BOTTOMLEFT",0,-1)

	Filters.Nontrivial=me:CreateFilterRow(theFrame,"  Non-Trivial")
	Filters.Nontrivial:SetPoint("TOP",Filters.Trivial,"BOTTOM",0,-1)

	Filters.Boss=me:CreateFilterRow(theFrame,"  Bosses")
	Filters.Boss:SetPoint("TOP",Filters.Nontrivial,"BOTTOM",0,-1)

	Filters.Unknown=me:CreateFilterRow(theFrame,"Unknown",true)
	Filters.Unknown:SetPoint("TOP",Filters.Boss,"BOTTOM",0,-1)

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.GlobalData=me:CreateSavedCheckbox("Global Data Collection",theFrame,"Data","GlobalData")
	theFrame.GlobalData:SetPoint("TOPLEFT",Filters.Unknown,"BOTTOMLEFT",0,-1)
	theFrame.GlobalData:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.GlobalDataCollect = true; else this:SetChecked(false); Recount.db.char.GlobalDataCollect = false; end end)
	
end


function me:SetBarTexture()
	local BarTextures=SM:List("statusbar")
	Recount:SetBarTextures(BarTextures[this.value])
	
	UIDropDownMenu_SetSelectedID(me.MiscOptions.StatusBarDropDown,this.value);
end

function me:BarTextureDropDown_Initialize()	
	local BarTextures=SM:List("statusbar")
	local LookingFor

	if not LookingFor then
		LookingFor=Recount.db.char.BarTexture
	end

	if not LookingFor then
		LookingFor="BantoBar"
	end

	for k,v in pairs(BarTextures) do
		info = {};
		info.text = v;
		info.value = k;
		info.func = me.SetBarTexture;
		UIDropDownMenu_AddButton(info);
		if v==LookingFor then
			LookingFor=k
		end
	end

	UIDropDownMenu_SetSelectedID(me.MiscOptions.StatusBarDropDown,LookingFor);
end

function me:SetSelectStatusBar(texture)
	if texture==nil then
		self:Hide()
		return
	end
	self.Text:SetText(texture)
	self.Texture:SetTexture(SM:Fetch("statusbar",texture))
	self.SetTo=texture
	self:Show()
end

function me:UpdateStatusBars()
	for _, v in pairs(me.TextureOptions.Rows) do
		if v.SetTo==Recount.db.char.BarTexture then
			v.Texture:SetVertexColor(0.2,0.9,0.2)
		else
			v.Texture:SetVertexColor(0.9,0.2,0.2)
		end
	end
end

function me:CreateSelectStatusBar(parent)
	local frame=CreateFrame("Frame",nil,parent)
	frame:SetHeight(13)
	frame:SetWidth(180)
	frame.Text=frame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	frame.Text:SetText("Temp")
	frame.Text:SetPoint("CENTER",frame,"CENTER")
	frame.Texture=frame:CreateTexture(nil,"BACKGROUND")
	frame.Texture:SetAllPoints(frame)
	frame.SetTexture=me.SetSelectStatusBar
	frame:EnableMouse()
	frame:SetScript("OnMouseDown",function() Recount:SetBarTextures(this.SetTo)
						me:UpdateStatusBars() end)
	return frame
end

function me:RefreshStatusBars()	
	local BarTextures=SM:List("statusbar")
	local size=table.getn(BarTextures)
	
	FauxScrollFrame_Update(me.TextureOptions.ScrollBar, size, 13, 12)
	local offset = FauxScrollFrame_GetOffset(me.TextureOptions.ScrollBar)
	
	for i=1,13 do
		me.TextureOptions.Rows[i]:SetTexture(BarTextures[i+offset])	
	end

	me:UpdateStatusBars()
end

function me:CreateTextureSelection(parent)
	me.TextureOptions=CreateFrame("Frame",nil,parent)

	local theFrame=me.TextureOptions
	local BarTextures=SM:List("statusbar")

	theFrame:SetHeight(200)
	theFrame:SetWidth(200)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",400,-34)

	--[[theFrame.Background=theFrame:CreateTexture(nil,"BACKGROUND")
	theFrame.Background:SetAllPoints(theFrame)
	theFrame.Background:SetTexture(0,0,0,0.3)]]

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetText("Bar Selection")
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)

	theFrame.Rows={}
	for i=1,13 do
		theFrame.Rows[i]=me:CreateSelectStatusBar(theFrame)
		theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",-8,-i*14-2)
		theFrame.Rows[i]:SetTexture(BarTextures[i])
	end
	me:UpdateStatusBars()

	if table.getn(BarTextures)<=13 then
		for i=1,13 do
			theFrame.Rows[i]:SetWidth(196)
			theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",0,-i*14-2)
		end
	end

	theFrame.ScrollBar=CreateFrame("SCROLLFRAME","Recount_Config_StatusBar_Scrollbar",theFrame,"FauxScrollFrameTemplate")
	theFrame.ScrollBar:SetScript("OnVerticalScroll", function() FauxScrollFrame_OnVerticalScroll(12, me.RefreshStatusBars) end)
	theFrame.ScrollBar:SetPoint("TOPLEFT",theFrame.Rows[1],"TOPLEFT")	
	theFrame.ScrollBar:SetPoint("BOTTOMRIGHT",theFrame.Rows[13],"BOTTOMRIGHT",-5,0)

	Recount:SetupScrollbar("Recount_Config_StatusBar_Scrollbar")

	me:RefreshStatusBars()
end


function me:SetSelectFont(font)
	if font==nil then
		self:Hide()
		return
	end
	self.Text:SetText(font)
	self.Text:SetFont(SM:Fetch("font",font),12)
	self.SetTo=font
	self:Show()
end

function me:UpdateFonts()
	for _, v in pairs(me.FontOptions.Rows) do
		if v.SetTo==Recount.db.char.Font then
			v.Texture:SetVertexColor(0.2,0.9,0.2)
		else
			v.Texture:SetVertexColor(0.9,0.2,0.2)
		end
	end
end

function me:CreateSelectFont(parent)
	local frame=CreateFrame("Frame",nil,parent)
	frame:SetHeight(13)
	frame:SetWidth(180)
	frame.Text=frame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	frame.Text:SetText("Temp")
	frame.Text:SetPoint("CENTER",frame,"CENTER")
	frame.Texture=frame:CreateTexture(nil,"BACKGROUND")
	frame.Texture:SetAllPoints(frame)
	frame.Texture:SetTexture(1,1,1,0.5)
	frame.SetFont=me.SetSelectFont
	frame:EnableMouse()
	frame:SetScript("OnMouseDown",function() Recount:SetFont(this.SetTo)
						me:UpdateFonts() end)
	return frame
end

function me:RefreshFonts()	
	local Fonts=SM:List("font")
	local size=table.getn(Fonts)
	
	FauxScrollFrame_Update(me.FontOptions.ScrollBar, size, 13, 12)
	local offset = FauxScrollFrame_GetOffset(me.FontOptions.ScrollBar)
	
	for i=1,13 do
		me.FontOptions.Rows[i]:SetFont(Fonts[i+offset])	
	end

	me:UpdateFonts()
end


function me:CreateFontSelection(parent)
	me.FontOptions=CreateFrame("Frame",nil,parent)

	local theFrame=me.FontOptions
	local Fonts=SM:List("font")

	theFrame:SetHeight(200)
	theFrame:SetWidth(200)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",200,-34)

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetText("Font Selection")
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)

	theFrame.Rows={}
	for i=1,13 do
		theFrame.Rows[i]=me:CreateSelectFont(theFrame)
		theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",-8,-i*14-2)
		theFrame.Rows[i]:SetFont(Fonts[i])
	end
	me:UpdateFonts()

	if table.getn(Fonts)<=13 then
		for i=1,13 do
			theFrame.Rows[i]:SetWidth(196)
			theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",0,-i*14-2)
		end
	end

	theFrame.ScrollBar=CreateFrame("SCROLLFRAME","Recount_Config_Fonts_Scrollbar",theFrame,"FauxScrollFrameTemplate")
	theFrame.ScrollBar:SetScript("OnVerticalScroll", function() FauxScrollFrame_OnVerticalScroll(12, me.RefreshFonts) end)
	theFrame.ScrollBar:SetPoint("TOPLEFT",theFrame.Rows[1],"TOPLEFT")	
	theFrame.ScrollBar:SetPoint("BOTTOMRIGHT",theFrame.Rows[13],"BOTTOMRIGHT",-5,0)

	Recount:SetupScrollbar("Recount_Config_Fonts_Scrollbar")

	me:RefreshFonts()
end

function me:SetupWindowOptions(parent)
	me.WindowOptions=CreateFrame("Frame",nil,parent)
	local theFrame=me.WindowOptions

	theFrame:SetHeight(200)
	theFrame:SetWidth(200)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",200,-34)

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetText("General Window Options")
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)

	theFrame.ResetWinButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.ResetWinButton:SetWidth(120)
	theFrame.ResetWinButton:SetHeight(24)
	theFrame.ResetWinButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",40,-20)
	theFrame.ResetWinButton:SetScript("OnClick",function() Recount:ResetPositions() end)
	theFrame.ResetWinButton:SetText("Reset Positions")
	
	slider = CreateFrame("Slider", "Recount_ConfigWindow_Scaling_Slider", theFrame,"OptionsSliderTemplate")
	theFrame.ScalingSlider=slider
	slider:SetOrientation("HORIZONTAL")
	slider:SetMinMaxValues(0.5, 1.5)
	slider:SetValueStep(0.05)
	slider:SetWidth(180)
	slider:SetHeight(16)
	slider:SetPoint("TOP", theFrame, "TOP", 0, -58)
	slider:SetScript("OnValueChanged",function() Recount.db.char.Scaling=math.floor(this:GetValue()*100+0.5)/100;getglobal(this:GetName().."Text"):SetText("Window Scaling: "..Recount.db.char.Scaling);Recount:ScaleWindows(Recount.db.char.Scaling) end)
	slider:SetScript("OnMouseUp", function() me:ScaleConfigWindow(Recount.db.char.Scaling) end)
	getglobal(slider:GetName().."High"):SetText("1.5");
	getglobal(slider:GetName().."Low"):SetText("0.5");
	getglobal(slider:GetName().."Text"):SetText("Window Scaling: "..Recount.db.char.Scaling)

	theFrame.ShowCurAndLast=me:CreateSavedCheckbox("Autoswitch Shown Fight",theFrame,"Window","ShowCurAndLast")
	theFrame.ShowCurAndLast:SetPoint("TOPLEFT",theFrame,"TOPLEFT",8,-82)
	theFrame.ShowCurAndLast:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.profile.Window.ShowCurAndLast = true; else this:SetChecked(false); Recount.db.profile.Window.ShowCurAndLast = false; end end)

	theFrame.LockWin=me:CreateSavedCheckbox("Lock Windows",theFrame,"Window","LockWin")
	theFrame.LockWin:SetPoint("TOPLEFT",theFrame,"TOPLEFT",8,-99)
	theFrame.LockWin:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.Locked = true; Recount:LockWindows(true) else this:SetChecked(false); Recount.db.char.Locked = false; Recount:LockWindows(false) end end)

end

function me:SetupVersionOptions(parent)
	me.VersionOptions=CreateFrame("Frame",nil,parent)
	local theFrame=me.VersionOptions
	theFrame:SetHeight(200)
	theFrame:SetWidth(200)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",400,-34)
	
	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetText("Recount Version")
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)
	
	theFrame.VersionText=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.VersionText:SetTextColor(1,1,1,1)
	theFrame.VersionText:SetText(Recount.Version)
	theFrame.VersionText:SetPoint("TOP",theFrame,"TOP",0,-26)

	theFrame.VerChkButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.VerChkButton:SetWidth(120)
	theFrame.VerChkButton:SetHeight(24)
	theFrame.VerChkButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",40,-44)
	theFrame.VerChkButton:SetScript("OnClick",function() Recount.ReportVersions() end)
	theFrame.VerChkButton:SetText("Check Versions")
end

function me:SetupRealtimeOptions(parent)
	me.RealtimeOptions=CreateFrame("Frame",nil,parent)
	local theFrame=me.RealtimeOptions
	theFrame:SetHeight(200)
	theFrame:SetWidth(200)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",400,-34)
	
	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetText("Global Realtime Windows")
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)

	theFrame.TitleRaid=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.TitleRaid:SetText("Raid")
	theFrame.TitleRaid:SetPoint("TOP",theFrame,"TOP",0,-26)
	
	theFrame.RDPSButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.RDPSButton:SetWidth(90)
	theFrame.RDPSButton:SetHeight(24)
	theFrame.RDPSButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",7,-40)
	theFrame.RDPSButton:SetScript("OnClick",function() Recount:CreateRealtimeWindow("!RAID","DAMAGE","Raid DPS") end)
	theFrame.RDPSButton:SetText("DPS")

	theFrame.RDTPSButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.RDTPSButton:SetWidth(90)
	theFrame.RDTPSButton:SetHeight(24)
	theFrame.RDTPSButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",102,-40)
	theFrame.RDTPSButton:SetScript("OnClick",function() Recount:CreateRealtimeWindow("!RAID","DAMAGETAKEN","Raid DTPS") end)
	theFrame.RDTPSButton:SetText("DTPS")

	theFrame.RHPSButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.RHPSButton:SetWidth(90)
	theFrame.RHPSButton:SetHeight(24)
	theFrame.RHPSButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",7,-66)
	theFrame.RHPSButton:SetScript("OnClick",function() Recount:CreateRealtimeWindow("!RAID","HEALING","Raid HPS") end)
	theFrame.RHPSButton:SetText("HPS")

	theFrame.RHTPSButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.RHTPSButton:SetWidth(90)
	theFrame.RHTPSButton:SetHeight(24)
	theFrame.RHTPSButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",102,-66)
	theFrame.RHTPSButton:SetScript("OnClick",function() Recount:CreateRealtimeWindow("!RAID","HEALINGTAKEN","Raid HTPS") end)
	theFrame.RHTPSButton:SetText("HTPS")

	theFrame.TitleRaid=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.TitleRaid:SetText("Network")
	theFrame.TitleRaid:SetPoint("TOP",theFrame,"TOP",0,-106)

	theFrame.FPSButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.FPSButton:SetWidth(90)
	theFrame.FPSButton:SetHeight(24)
	theFrame.FPSButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",7,-120)
	theFrame.FPSButton:SetScript("OnClick",function() Recount:CreateRealtimeWindow("FPS","FPS","") end)
	theFrame.FPSButton:SetText("FPS")

	theFrame.LATButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.LATButton:SetWidth(90)
	theFrame.LATButton:SetHeight(24)
	theFrame.LATButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",102,-120)
	theFrame.LATButton:SetScript("OnClick",function() Recount:CreateRealtimeWindow("Latency","LAG","") end)
	theFrame.LATButton:SetText("Latency")

	theFrame.UPTButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.UPTButton:SetWidth(90)
	theFrame.UPTButton:SetHeight(24)
	theFrame.UPTButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",7,-146)
	theFrame.UPTButton:SetScript("OnClick",function() Recount:CreateRealtimeWindow("Upstream Traffic","UP_TRAFFIC","") end)
	theFrame.UPTButton:SetText("Up Traffic")

	theFrame.DOTButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.DOTButton:SetWidth(90)
	theFrame.DOTButton:SetHeight(24)
	theFrame.DOTButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",102,-146)
	theFrame.DOTButton:SetScript("OnClick",function() Recount:CreateRealtimeWindow("Downstream Traffic","DOWN_TRAFFIC","") end)
	theFrame.DOTButton:SetText("Down Traffic")

	theFrame.BWButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.BWButton:SetWidth(90)
	theFrame.BWButton:SetHeight(24)
	theFrame.BWButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",55,-172)
	theFrame.BWButton:SetScript("OnClick",function() Recount:CreateRealtimeWindow("Bandwidth Available","AVAILABLE_BANDWIDTH","") end)
	theFrame.BWButton:SetText("Bandwidth")
end

function me:SetupMiscOptions(parent)
	me.MiscOptions=CreateFrame("Frame",nil,parent)
	local theFrame=me.MiscOptions

	theFrame:SetHeight(200)
	theFrame:SetWidth(200)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",200,-34)

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetText("Data Options")
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)
-- Elsia: Sacrificed these two lines to make space for Sync toggle, edit: desacrified in the process of ditching save/revert
	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Sync=me:CreateSavedCheckbox("Sync Data",theFrame,"Data","Sync")
	theFrame.Sync:SetPoint("TOPLEFT",theFrame,"TOPLEFT",8,-16)
	theFrame.Sync:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount:ConfigComm(); Recount.db.char.EnableSync = true  else this:SetChecked(false); Recount.db.char.EnableSync = false; Recount:FreeComm() end end)

	theFrame.Autodelete=me:CreateSavedCheckbox("Autodelete Time Data",theFrame,"Data","AutodeleteTime")
	theFrame.Autodelete:SetPoint("TOPLEFT",theFrame,"TOPLEFT",8,-33)
	theFrame.Autodelete:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.AutoDelete = true; else this:SetChecked(false); Recount.db.char.AutoDelete = false; end end)

	theFrame.AutodeleteC=me:CreateSavedCheckbox("Autodelete Combatants",theFrame,"Data","AutodeleteChars")
	theFrame.AutodeleteC:SetPoint("TOPLEFT",theFrame,"TOPLEFT",8,-50)	
	theFrame.AutodeleteC:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.AutoDeleteCombatants = true; else this:SetChecked(false); Recount.db.char.AutoDeleteCombatants = false; end end)

	local slider = CreateFrame("Slider", "Recount_ConfigWindow_TimeData_Slider", theFrame,"OptionsSliderTemplate")
	theFrame.TimeDataSlider=slider
	slider:SetOrientation("HORIZONTAL")
	slider:SetMinMaxValues(30, 300)
	slider:SetValueStep(5)
	slider:SetValue(30)
	slider:SetWidth(180)
	slider:SetHeight(16)
	slider:SetPoint("TOP", theFrame, "TOP", 0, -83)
	slider:SetScript("OnValueChanged",function() getglobal(this:GetName().."Text"):SetText("Save Data For: "..this:GetValue().. " Minutes"); Recount.db.char.AutoDeleteTime=this:GetValue() end)
	getglobal(slider:GetName().."High"):SetText("300");
	getglobal(slider:GetName().."Low"):SetText("30");
	getglobal(slider:GetName().."Text"):SetText("Save Data For: "..slider:GetValue().. " Minutes")

	theFrame.CombatLogRange=me:CreateSavedCheckbox("Set Combat Log Range",theFrame,"Data","SetCombatLog")
	theFrame.CombatLogRange:SetPoint("TOPLEFT",theFrame,"TOPLEFT",8,-113)
	theFrame.CombatLogRange:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.UseCombatLogRange = true;  else this:SetChecked(false); Recount.db.char.UseCombatLogRange = false;  end if Recount.db.char.UseCombatLogRange then Recount:SetLogRange(Recount.db.char.CombatLogRange) else Recount:SetLogRange(200) end end) -- Elsia: Changed default to 200

	local slider = CreateFrame("Slider", "Recount_ConfigWindow_CombatLogRange_Slider", theFrame,"OptionsSliderTemplate")
	theFrame.CombatLogRangeSlider=slider
	slider:SetOrientation("HORIZONTAL")
	slider:SetMinMaxValues(40, 200)
	slider:SetValueStep(5)
	slider:SetValue(40)
	slider:SetWidth(180)
	slider:SetHeight(16)
	slider:SetPoint("TOP", theFrame, "TOP", 0, -147)
	slider:SetScript("OnValueChanged",function() getglobal(this:GetName().."Text"):SetText("Combat Log Range: "..this:GetValue().. " Yds"); Recount.db.char.CombatLogRange=this:GetValue() end)
	getglobal(slider:GetName().."High"):SetText("200");
	getglobal(slider:GetName().."Low"):SetText("40");
	getglobal(slider:GetName().."Text"):SetText("Combat Log Range: "..slider:GetValue().. " Yds")

	theFrame.MergePets=me:CreateSavedCheckbox("Merge Pets w/ Owners",theFrame,"Data","MergePets")
	theFrame.MergePets:SetPoint("TOPLEFT",theFrame,"TOPLEFT",8,-177)
	theFrame.MergePets:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.MergePets = true; Recount.db.char.Filters.Show["Pet"]=false else this:SetChecked(false); Recount.db.char.MergePets = false; Recount.db.char.Filters.Show["Pet"]=true end me.FilterOptions.Filters.Pet.ShowData:SetChecked(Recount.db.char.Filters.Show["Pet"]); Recount:FullRefreshMainWindow() end)

	theFrame.FixLog=me:CreateSavedCheckbox("Fix Ambiguous Log Strings",theFrame,"Data","FixLogStrings")
	theFrame.FixLog:SetPoint("TOPLEFT",theFrame,"TOPLEFT",8,-193)
	theFrame.FixLog:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.UseFixLog = true; else this:SetChecked(false); Recount.db.char.UseFixLog = false; end end)
end

function me:SetupButtonOptions(parent)
	me.ButtonOptions=CreateFrame("Frame",nil,parent)
	local theFrame=me.ButtonOptions

	theFrame:SetHeight(196)
	theFrame:SetWidth(196)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",0,-34)
	--Reset
	--File
	--Config
	--Report

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetText("Main Window Options")
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)

	theFrame.ButtonsTitle=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.ButtonsTitle:SetText("Show Buttons")
	theFrame.ButtonsTitle:SetPoint("TOP",theFrame,"TOPLEFT",100,-16)
	
	theFrame.ReportButton=CreateFrame("CheckButton",nil,theFrame)
	me:ConfigureCheckbox(theFrame.ReportButton)
	theFrame.ReportButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",8,-18-16)
	theFrame.ReportButton:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.MainWindow.Buttons.ReportButton = true; Recount:SetupMainWindowButtons() else this:SetChecked(false); Recount.db.char.MainWindow.Buttons.ReportButton = false; Recount:SetupMainWindowButtons() end end)

	theFrame.Report_Icon=theFrame:CreateTexture(nil,"OVERLAY")
	theFrame.Report_Icon:SetWidth(16)
	theFrame.Report_Icon:SetHeight(16)
	theFrame.Report_Icon:SetTexture("Interface\\Buttons\\UI-GuildButton-MOTD-Up.blp")
	theFrame.Report_Icon:SetPoint("LEFT",theFrame.ReportButton,"RIGHT",2,0)

	theFrame.Report_Text=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Report_Text:SetText("Report")
	theFrame.Report_Text:SetPoint("LEFT",theFrame.Report_Icon,"RIGHT",2,0)

	theFrame.FileButton=CreateFrame("CheckButton",nil,theFrame)
	me:ConfigureCheckbox(theFrame.FileButton)
	theFrame.FileButton:SetPoint("TOP",theFrame.ReportButton,"BOTTOM",0,-2)
	theFrame.FileButton:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.MainWindow.Buttons.FileButton = true; Recount:SetupMainWindowButtons() else this:SetChecked(false); Recount.db.char.MainWindow.Buttons.FileButton = false; Recount:SetupMainWindowButtons() end end)

	theFrame.File_Icon=theFrame:CreateTexture(nil,"OVERLAY")
	theFrame.File_Icon:SetWidth(16)
	theFrame.File_Icon:SetHeight(16)
	theFrame.File_Icon:SetTexture("Interface\\Buttons\\UI-GuildButton-PublicNote-Up.blp")
	theFrame.File_Icon:SetPoint("LEFT",theFrame.FileButton,"RIGHT",2,0)

	theFrame.File_Text=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.File_Text:SetText("File")
	theFrame.File_Text:SetPoint("LEFT",theFrame.File_Icon,"RIGHT",2,0)

	theFrame.ConfigButton=CreateFrame("CheckButton",nil,theFrame)
	me:ConfigureCheckbox(theFrame.ConfigButton)
	theFrame.ConfigButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",100,-18-16)
	theFrame.ConfigButton:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.MainWindow.Buttons.ConfigButton = true; Recount:SetupMainWindowButtons() else this:SetChecked(false); Recount.db.char.MainWindow.Buttons.ConfigButton = false; Recount:SetupMainWindowButtons() end end)

	theFrame.Config_Icon=theFrame:CreateTexture(nil,"OVERLAY")
	theFrame.Config_Icon:SetWidth(16)
	theFrame.Config_Icon:SetHeight(16)
	theFrame.Config_Icon:SetTexture("Interface\\Addons\\Recount\\Textures\\icon-config")
	theFrame.Config_Icon:SetPoint("LEFT",theFrame.ConfigButton,"RIGHT",2,0)

	theFrame.Config_Text=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Config_Text:SetText("Config")
	theFrame.Config_Text:SetPoint("LEFT",theFrame.Config_Icon,"RIGHT",2,0)

	theFrame.ResetButton=CreateFrame("CheckButton",nil,theFrame)
	me:ConfigureCheckbox(theFrame.ResetButton)
	theFrame.ResetButton:SetPoint("TOP",theFrame.ConfigButton,"BOTTOM",0,-2)
	theFrame.ResetButton:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.MainWindow.Buttons.ResetButton = true; Recount:SetupMainWindowButtons() else this:SetChecked(false); Recount.db.char.MainWindow.Buttons.ResetButton = false; Recount:SetupMainWindowButtons() end end)

	theFrame.Reset_Icon=theFrame:CreateTexture(nil,"OVERLAY")
	theFrame.Reset_Icon:SetWidth(16)
	theFrame.Reset_Icon:SetHeight(16)
	theFrame.Reset_Icon:SetTexture("Interface\\Addons\\Recount\\Textures\\icon-reset")
	theFrame.Reset_Icon:SetPoint("LEFT",theFrame.ResetButton,"RIGHT",2,0)

	theFrame.Reset_Text=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Reset_Text:SetText("Reset")
	theFrame.Reset_Text:SetPoint("LEFT",theFrame.Reset_Icon,"RIGHT",2,0)

	theFrame.LeftButton=CreateFrame("CheckButton",nil,theFrame) -- Elsia: Added paging icon toggle support
	me:ConfigureCheckbox(theFrame.LeftButton)
	theFrame.LeftButton:SetPoint("TOP",theFrame.FileButton,"BOTTOM",0,-2)
	theFrame.LeftButton:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.MainWindow.Buttons.LeftButton = true; Recount:SetupMainWindowButtons() else this:SetChecked(false); Recount.db.char.MainWindow.Buttons.LeftButton = false; Recount:SetupMainWindowButtons() end end)

	theFrame.Left_Icon=theFrame:CreateTexture(nil,"OVERLAY")
	theFrame.Left_Icon:SetWidth(16)
	theFrame.Left_Icon:SetHeight(16)
	theFrame.Left_Icon:SetTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up.blp")
	theFrame.Left_Icon:SetPoint("LEFT",theFrame.LeftButton,"RIGHT",2,0)

	theFrame.Left_Text=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Left_Text:SetText("Previous")
	theFrame.Left_Text:SetPoint("LEFT",theFrame.Left_Icon,"RIGHT",2,0)

	theFrame.RightButton=CreateFrame("CheckButton",nil,theFrame) -- Elsia: Added paging icon toggle support
	me:ConfigureCheckbox(theFrame.RightButton)
	theFrame.RightButton:SetPoint("TOP",theFrame.ResetButton,"BOTTOM",0,-2)
	theFrame.RightButton:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.MainWindow.Buttons.RightButton = true; Recount:SetupMainWindowButtons() else this:SetChecked(false); Recount.db.char.MainWindow.Buttons.RightButton = false; Recount:SetupMainWindowButtons() end end)

	theFrame.Right_Icon=theFrame:CreateTexture(nil,"OVERLAY")
	theFrame.Right_Icon:SetWidth(16)
	theFrame.Right_Icon:SetHeight(16)
	theFrame.Right_Icon:SetTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up.blp")
	theFrame.Right_Icon:SetPoint("LEFT",theFrame.RightButton,"RIGHT",2,0)

	theFrame.Right_Text=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Right_Text:SetText("Next")
	theFrame.Right_Text:SetPoint("LEFT",theFrame.Right_Icon,"RIGHT",2,0)

	local slider = CreateFrame("Slider", "Recount_ConfigWindow_RowHeight_Slider", theFrame,"OptionsSliderTemplate")
	theFrame.RowHeightSlider=slider
	slider:SetOrientation("HORIZONTAL")
	slider:SetMinMaxValues(8, 24)
	slider:SetValueStep(1)
	slider:SetWidth(180)
	slider:SetHeight(16)
	slider:SetPoint("TOP", theFrame, "TOP", 0, -96-16) -- Elsia: TODO this number will need adjusting to accommodate the paging config change
	slider:SetScript("OnValueChanged",function() getglobal(this:GetName().."Text"):SetText("Row Height: "..this:GetValue());Recount.db.char.MainWindow.RowHeight=this:GetValue();Recount:BarsChanged() end)
	getglobal(slider:GetName().."High"):SetText("24");
	getglobal(slider:GetName().."Low"):SetText("8");
	getglobal(slider:GetName().."Text"):SetText("Row Height: "..slider:GetValue())

	slider = CreateFrame("Slider", "Recount_ConfigWindow_RowSpacing_Slider", theFrame,"OptionsSliderTemplate")
	theFrame.RowSpacingSlider=slider
	slider:SetOrientation("HORIZONTAL")
	slider:SetMinMaxValues(0, 4)
	slider:SetValueStep(1)
	slider:SetWidth(180)
	slider:SetHeight(16)
	slider:SetPoint("TOP", theFrame, "TOP", 0, -130-16)
	slider:SetScript("OnValueChanged",function() getglobal(this:GetName().."Text"):SetText("Row Spacing: "..this:GetValue());Recount.db.char.MainWindow.RowSpacing=this:GetValue();Recount:BarsChanged() end)
	getglobal(slider:GetName().."High"):SetText("4");
	getglobal(slider:GetName().."Low"):SetText("0");
	getglobal(slider:GetName().."Text"):SetText("Row Spacing: "..slider:GetValue())

	theFrame.AutoHide=CreateFrame("CheckButton",nil,theFrame)
	me:ConfigureCheckbox(theFrame.AutoHide)
	theFrame.AutoHide:SetPoint("TOPLEFT",theFrame,"TOPLEFT",12,-160-16)
	theFrame.AutoHide:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.MainWindow.AutoHide = true; else this:SetChecked(false); Recount.db.char.MainWindow.AutoHide = false; end end)

	theFrame.AutohideText=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.AutohideText:SetText("Autohide On Combat")
	theFrame.AutohideText:SetPoint("LEFT",theFrame.AutoHide,"RIGHT",8,0)

	theFrame.ShowSB=CreateFrame("CheckButton",nil,theFrame)
	me:ConfigureCheckbox(theFrame.ShowSB)
	theFrame.ShowSB:SetPoint("TOPLEFT",theFrame,"TOPLEFT",12,-177-16)
	theFrame.ShowSB:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true); Recount.db.char.MainWindow.ShowScrollbar = true; else this:SetChecked(false); Recount.db.char.MainWindow.ShowScrollbar = false; end Recount:RefreshMainWindow() end)

	theFrame.ShowSBText=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.ShowSBText:SetText("Show Scrollbar")
	theFrame.ShowSBText:SetPoint("LEFT",theFrame.ShowSB,"RIGHT",8,0)
	
end




function me:ScaleConfigWindow(scale)
	local pointNum=me.ConfigWindow:GetNumPoints()
	local curScale=me.ConfigWindow:GetScale();
	local points={}
	for i=1,pointNum,1 do
		points[i]={};
		points[i][1], points[i][2], points[i][3], points[i][4], points[i][5]=me.ConfigWindow:GetPoint(i)
		points[i][4]=points[i][4]*curScale/scale;
		points[i][5]=points[i][5]*curScale/scale;
	end

	me.ConfigWindow:ClearAllPoints()
	for i=1,pointNum,1 do
		me.ConfigWindow:SetPoint(points[i][1],points[i][2],points[i][3],points[i][4],points[i][5]);
	end

	me.ConfigWindow:SetScale(scale)
end

function me:HideOptions()
	me.ConfigWindow.Data:Hide()
	me.ConfigWindow.Data.Tab.Background:SetVertexColor(1.0,0.2,0.2)
	me.ConfigWindow.Appearance:Hide()
	me.ConfigWindow.Appearance.Tab.Background:SetVertexColor(1.0,0.2,0.2)
	me.ConfigWindow.Window:Hide()
	me.ConfigWindow.Window.Tab.Background:SetVertexColor(1.0,0.2,0.2)
	me.ConfigWindow.ColorOpt:Hide()
	me.ConfigWindow.ColorOpt.Tab.Background:SetVertexColor(1.0,0.2,0.2)
end


function me:CreateDataOptions(parent)
	local theFrame=CreateFrame("FRAME",nil,parent)
	parent.Data=theFrame

	theFrame:SetWidth(600)
	theFrame:SetHeight(200)
	theFrame:SetPoint("TOP",parent,"TOP",0,-22)

	local Tab=CreateFrame("FRAME",nil,parent)
	parent.Data.Tab=Tab

	Tab:SetWidth(100)
	Tab:SetHeight(18)
	Tab:SetPoint("TOPLEFT", parent, "TOPLEFT",4,-35)
	Tab:EnableMouse(true)
	Tab:SetScript("OnMouseDown",function() me:HideOptions();theFrame:Show();this.Background:SetVertexColor(0.2,1.0,0.2) end)
	Tab.Text=Tab:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Tab.Text:SetPoint("CENTER",Tab,"CENTER")
	Tab.Text:SetText("Data")
	Tab.Background=Tab:CreateTexture(nil,"BACKGROUND")
	Tab.Background:SetTexture(1,1,1,0.3)
	Tab.Background:SetVertexColor(0.2,1.0,0.2)
	Tab.Background:SetAllPoints(Tab)


	me:SetupFilterOptions(theFrame)
	me:SetupMiscOptions(theFrame)
	me:SetupVersionOptions(theFrame)
end

function me:CreateAppearanceOptions(parent)
	local theFrame=CreateFrame("FRAME",nil,parent)
	parent.Appearance=theFrame

	theFrame:SetWidth(600)
	theFrame:SetHeight(200)
	theFrame:SetPoint("TOP",parent,"TOP",0,-22)

	local Tab=CreateFrame("FRAME",nil,parent)
	parent.Appearance.Tab=Tab

	Tab:SetWidth(100)
	Tab:SetHeight(18)
	Tab:SetPoint("TOPLEFT", parent, "TOPLEFT",208,-35)
	Tab:EnableMouse(true)
	Tab:SetScript("OnMouseDown",function() me:HideOptions();theFrame:Show();this.Background:SetVertexColor(0.2,1.0,0.2) end)
	Tab.Text=Tab:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Tab.Text:SetPoint("CENTER",Tab,"CENTER")
	Tab.Text:SetText("Appearance")
	Tab.Background=Tab:CreateTexture(nil,"BACKGROUND")
	Tab.Background:SetTexture(1,1,1,0.3)
	Tab.Background:SetVertexColor(1.0,0.2,0.2)
	Tab.Background:SetAllPoints(Tab)

	me:CreateTextureSelection(theFrame)
	me:CreateFontSelection(theFrame)
	theFrame:Hide()
end

function me:CreateColorOptions(parent)
	local theFrame=CreateFrame("FRAME",nil,parent)
	parent.ColorOpt=theFrame

	theFrame:SetWidth(600)
	theFrame:SetHeight(200)
	theFrame:SetPoint("TOP",parent,"TOP",0,-22)

	local Tab=CreateFrame("FRAME",nil,parent)
	parent.ColorOpt.Tab=Tab

	Tab:SetWidth(100)
	Tab:SetHeight(18)
	Tab:SetPoint("TOPLEFT", parent, "TOPLEFT",310,-35) -- Elsia: Check tab offset
	Tab:EnableMouse(true)
	Tab:SetScript("OnMouseDown",function() me:HideOptions();theFrame:Show();this.Background:SetVertexColor(0.2,1.0,0.2) end)
	Tab.Text=Tab:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Tab.Text:SetPoint("CENTER",Tab,"CENTER")
	Tab.Text:SetText("Color")
	Tab.Background=Tab:CreateTexture(nil,"BACKGROUND")
	Tab.Background:SetTexture(1,1,1,0.3)
	Tab.Background:SetVertexColor(1.0,0.2,0.2)
	Tab.Background:SetAllPoints(Tab)

	me:CreateWindowColorSelection(theFrame)
	me:CreateClassColorSelection(theFrame)
	theFrame:Hide()
end



function me:CreateWindowOptions(parent)
	local theFrame=CreateFrame("FRAME",nil,parent)
	parent.Window=theFrame

	local Tab=CreateFrame("FRAME",nil,parent)
	parent.Window.Tab=Tab

	Tab:SetWidth(100)
	Tab:SetHeight(18)
	Tab:SetPoint("TOPLEFT", parent, "TOPLEFT",106,-35)
	Tab:EnableMouse(true)
	Tab:SetScript("OnMouseDown",function() me:HideOptions();theFrame:Show();this.Background:SetVertexColor(0.2,1.0,0.2) end)
	Tab.Text=Tab:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Tab.Text:SetPoint("CENTER",Tab,"CENTER")
	Tab.Text:SetText("Window")
	Tab.Background=Tab:CreateTexture(nil,"BACKGROUND")
	Tab.Background:SetTexture(1,1,1,0.3)
	Tab.Background:SetVertexColor(1.0,0.2,0.2)
	Tab.Background:SetAllPoints(Tab)

	theFrame:SetWidth(600)
	theFrame:SetHeight(200)
	theFrame:SetPoint("TOP",parent,"TOP",0,-22)

	me:SetupButtonOptions(theFrame)
	me:SetupWindowOptions(theFrame)
	me:SetupRealtimeOptions(theFrame)
	theFrame:Hide()
end

function me:CreateConfigWindow()
	me.ConfigWindow=Recount:CreateFrame("Recount_ConfigWindow","Config Recount",286,600)

	local theFrame=me.ConfigWindow
	
-- Elsia: This is where the save and revert buttons died
--	theFrame.SaveButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
--	theFrame.SaveButton:SetWidth(90)
--	theFrame.SaveButton:SetHeight(24)
--	theFrame.SaveButton:SetPoint("BOTTOMRIGHT",theFrame,"BOTTOM",-2,4)
--	theFrame.SaveButton:SetScript("OnClick",function() me:SaveConfig() end)
--	theFrame.SaveButton:SetText("Save")

--	theFrame.RevertButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
--	theFrame.RevertButton:SetWidth(90)
--	theFrame.RevertButton:SetHeight(24)
--	theFrame.RevertButton:SetPoint("BOTTOMLEFT",theFrame,"BOTTOM",2,4)
--	theFrame.RevertButton:SetScript("OnClick",function() me:LoadConfig() end)
--	theFrame.RevertButton:SetText("Revert")
	
	Recount.Colors:RegisterTexture("Other Windows","Title",Graph:DrawLine(theFrame,200,12,200,233,24,{0.5,0.0,0.0,1.0},"ARTWORK"),{r=0.5,g=0.5,b=0.5,a=1}) -- Elsia: Changed 32->12 for longer separators given no save/revert
	Recount.Colors:RegisterTexture("Other Windows","Title",Graph:DrawLine(theFrame,400,12,400,233,24,{0.5,0.0,0.0,1.0},"ARTWORK"),{r=0.5,g=0.5,b=0.5,a=1})
	Recount.Colors:RegisterTexture("Other Windows","Title",Graph:DrawLine(theFrame,2,233,598,233,24,{0.5,0.0,0.0,1.0},"ARTWORK"),{r=0.5,g=0.5,b=0.5,a=1})

	theFrame:Hide()

	me:CreateDataOptions(theFrame)
	me:CreateAppearanceOptions(theFrame)
	me:CreateWindowOptions(theFrame)
	me:CreateColorOptions(theFrame)
	
	--me:CreateColorSelection(theFrame)

	--Need to add it to our window ordering system
	Recount:AddWindow(theFrame)
end

function me:LoadConfig()
	for k, v in pairs(me.FilterOptions.Filters) do
		v.ShowData:SetChecked(Recount.db.char.Filters.Show[k])
		v.RecordData:SetChecked(Recount.db.char.Filters.Data[k])
		v.RecordTime:SetChecked(Recount.db.char.Filters.TimeData[k])
		v.TrackDeaths:SetChecked(Recount.db.char.Filters.TrackDeaths[k])		
	end

	me.FilterOptions.GlobalData:SetChecked(Recount.db.char.GlobalDataCollect)
	
	for k, v in pairs(Recount.db.char.MainWindow.Buttons) do
		me.ButtonOptions[k]:SetChecked(v)
	end

	me.ButtonOptions.RowHeightSlider:SetValue(Recount.db.char.MainWindow.RowHeight)
	me.ButtonOptions.RowSpacingSlider:SetValue(Recount.db.char.MainWindow.RowSpacing)
	me.ButtonOptions.AutoHide:SetChecked(Recount.db.char.MainWindow.AutoHide)
	me.ButtonOptions.ShowSB:SetChecked(Recount.db.char.MainWindow.ShowScrollbar)
	
	me.WindowOptions.ScalingSlider:SetValue(Recount.db.char.Scaling)
	me.WindowOptions.ShowCurAndLast:SetChecked(Recount.db.profile.Window.ShowCurAndLast)
	me.WindowOptions.LockWin:SetChecked(Recount.db.char.Locked)

	me.MiscOptions.Sync:SetChecked(Recount.db.char.EnableSync)
	me.MiscOptions.Autodelete:SetChecked(Recount.db.char.AutoDelete)
	me.MiscOptions.AutodeleteC:SetChecked(Recount.db.char.AutoDeleteCombatants)
	me.MiscOptions.MergePets:SetChecked(Recount.db.char.MergePets)
	me.MiscOptions.TimeDataSlider:SetValue(Recount.db.char.AutoDeleteTime)
	me.MiscOptions.CombatLogRange:SetChecked(Recount.db.char.UseCombatLogRange)
	me.MiscOptions.CombatLogRangeSlider:SetValue(Recount.db.char.CombatLogRange)
	me.MiscOptions.FixLog:SetChecked(Recount.db.char.UseFixLog)

	me:ScaleConfigWindow(Recount.db.char.Scaling)
end

function me:SaveFilterConfig()
	for k, v in pairs(me.FilterOptions.Filters) do
		Recount.db.char.Filters.Show[k]=v.ShowData:GetChecked()==1
		Recount.db.char.Filters.Data[k]=v.RecordData:GetChecked()==1
		Recount.db.char.Filters.TimeData[k]=v.RecordTime:GetChecked()==1
		Recount.db.char.Filters.TrackDeaths[k]=v.TrackDeaths:GetChecked()==1
	end
	Recount:FullRefreshMainWindow()
end

--[ Elsia: This is now obsolete, RIP save
function me:SaveConfig()
	for k, v in pairs(me.FilterOptions.Filters) do
		Recount.db.char.Filters.Show[k]=v.ShowData:GetChecked()==1
		Recount.db.char.Filters.Data[k]=v.RecordData:GetChecked()==1
		Recount.db.char.Filters.TimeData[k]=v.RecordTime:GetChecked()==1
		Recount.db.char.Filters.TrackDeaths[k]=v.TrackDeaths:GetChecked()==1
	end

	for k, v in pairs(Recount.db.char.MainWindow.Buttons) do
		Recount.db.char.MainWindow.Buttons[k]=(me.ButtonOptions[k]:GetChecked()==1)
	end
	Recount.db.char.MainWindow.AutoHide=me.ButtonOptions.AutoHide:GetChecked()==1

	Recount.db.char.EnableSync=me.MiscOptions.Sync:GetChecked()==1
	Recount.db.char.AutoDelete=me.MiscOptions.Autodelete:GetChecked()==1
	Recount.db.char.AutoDeleteCombatants=me.MiscOptions.AutodeleteC:GetChecked()==1
	Recount.db.char.AutoDeleteTime=me.MiscOptions.TimeDataSlider:GetValue()
	
	if (not Recount.db.char.MergePets) and (me.MiscOptions.MergePets:GetChecked()==1) then -- Elsia: Toggle Pet display if merge changed
		Recount.db.char.Filters.Show["Pet"]=false
		me.FilterOptions.Filters.Pet.ShowData:SetChecked(Recount.db.char.Filters.Show["Pet"])
	elseif Recount.db.char.MergePets and (me.MiscOptions.MergePets:GetChecked()~=1) then
		Recount.db.char.Filters.Show["Pet"]=true
		me.FilterOptions.Filters.Pet.ShowData:SetChecked(Recount.db.char.Filters.Show["Pet"])
	end

	me:ScaleConfigWindow(Recount.db.char.Scaling) -- Elsia: Refresh display as we changed options

	-- Elsia: Leave it alone otherwise, people can show the pets that way if they want to, but merge will force display on and off when changed.
	
	Recount.db.char.MergePets=me.MiscOptions.MergePets:GetChecked()==1
	
	Recount.db.char.UseFixLog=me.MiscOptions.FixLog:GetChecked()==1

	Recount.db.profile.Window.ShowCurAndLast=me.WindowOptions.ShowCurAndLast:GetChecked()==1

	Recount.db.char.UseCombatLogRange=me.MiscOptions.CombatLogRange:GetChecked()==1
	Recount.db.char.CombatLogRange=me.MiscOptions.CombatLogRangeSlider:GetValue()

	if Recount.db.char.UseCombatLogRange then
		Recount:SetLogRange(Recount.db.char.CombatLogRange)
	else
		Recount:SetLogRange(40)  -- Elsia: Was SetLogRangeDefault() but the function doesn't exist. No bother, 40 is default.
	end


	Recount.MainWindow.DispTableSorted={}
	Recount.MainWindow.DispTableLookup={}

	Recount:SetupMainWindowButtons()
	Recount:RefreshMainWindow()
end
--]

function Recount:ShowConfig()
	if type(me.ConfigWindow)=="nil" then
		me:CreateConfigWindow()
	end
	me:LoadConfig()
	me.ConfigWindow:Show()
end


function Recount:ConfigWindowStatus()
	if me.ConfigWindow.Below then
		below = me.ConfigWindow.Below:GetName()
	else
		below = "(nil)"
	end

	if me.ConfigWindow.Above then
		above = me.ConfigWindow.Above:GetName()
	else
		above = "(nil)"
	end

	Recount:Print(below.." Config "..above)
end

